==> iRRAM_2003_03.tar.gz: iRRAM_2003_03/examples/itsyst.cc <==
#include "iRRAM.h"

/* Compute iterated system x=3.75*x*(1-x) (Kulisch) */

void itsyst_double(int count){
  double x = 0.5;
  double c = 3.75;

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rprintf("%13.10f",x); rprintf(" %d\n",i);
    }
   x= c*x*(1-x);
  }
}


void itsyst_RATIONAL(int count){

  RATIONAL x = RATIONAL(1)/RATIONAL(2);
  RATIONAL c = RATIONAL(375)/RATIONAL(100);

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(REAL(x),18); rprintf(" %d\n",i);
    }
    x= c*x*(RATIONAL(1)-x);
  }
}



void itsyst_REAL(int count){

  REAL x = 0.5;
  REAL c = 3.75;

// 0.5 and 3.75 can be represented exactly as double, so 
// c=3.75 and c="3.75" are the same, but the latter is a lot slower!

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(x,18); rprintf(" %d\n",i);
    }
    x= c*x*(REAL(1)-x);
  }
}



// Using addtional information like the Lipschitz condition
// for the iteration improves on the error propagation.
// Although we have to evaluate the Lipschitz condition,
// the improvements due to the better error propagation 
// are significatn (speedup about 70%).
 
REAL iteration(const REAL& x) {return  REAL(3.75)*x*(1-x);}
REAL iteration_lip(const REAL& x) {return  REAL(3.75) - REAL(7.5)*x;}

void itsyst_REAL_with_lipschitz_condition(int count){

  REAL x = 0.5;

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(x,18); rprintf(" %d\n",i);
    }
    x= lipschitz(iteration,iteration_lip,x);
  }
}


void itsyst_with_INTEGER(int count){


  REAL x = 0.5;

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(x,18); rprintf(" %d\n",i);
    }
    x= (x*(1l-x)*375l)/100l;
  }
}


void compute(){
  int test,count;
  rprintf("\nIterated functions system: x = 3.75*x*(1-x)\n");
  rprintf("\nHow to compute (1=double, 2=RATIONAL, 3=REAL, 4=with_INTEGER, 5=with_Lipschitz) : ");
  rscanf("%d",&test);
  rprintf("How many values: ");
  rscanf("%d",&count);
  rprintf("\n");
  switch (test) {
     case 1:;
       itsyst_double(count);
     break;
     case 2:;
       itsyst_RATIONAL(count);
     break;
     case 3:;
       itsyst_REAL(count);
     break;
     case 4:;
       itsyst_with_INTEGER(count);
     break;
     case 5:;
       itsyst_REAL_with_lipschitz_condition(count);
     break;
  }
  rprintf("\n");
};

==> iRRAM_2003_08.tar.gz: iRRAM_2003_08/examples/itsyst.cc <==
#include "iRRAM.h"

/* Compute iterated system x=3.75*x*(1-x) (Kulisch) */

void itsyst_double(int count){
  double x = 0.5;
  double c = 3.75;

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rprintf("%13.10f",x); rprintf(" %d\n",i);
    }
   x= c*x*(1-x);
  }
}


void itsyst_RATIONAL(int count){

  RATIONAL x = RATIONAL(1)/RATIONAL(2);
  RATIONAL c = RATIONAL(375)/RATIONAL(100);

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(REAL(x),18); rprintf(" %d\n",i);
    }
    x= c*x*(RATIONAL(1)-x);
  }
}



void itsyst_REAL(int count){

  REAL x = 0.5;
  REAL c = 3.75;

// 0.5 and 3.75 can be represented exactly as double, so 
// c=3.75 and c="3.75" are the same, but the latter is a lot slower!

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(x,18); rprintf(" %d\n",i);
    }
    x= c*x*(REAL(1)-x);
  }
}



// Using addtional information like the Lipschitz condition
// for the iteration improves on the error propagation.
// Although we have to evaluate the Lipschitz condition,
// the improvements due to the better error propagation 
// are significatn (speedup about 70%).
 
REAL iteration(const REAL& x) {return  REAL(3.75)*x*(1-x);}
REAL iteration_lip(const REAL& x) {return  REAL(3.75) - REAL(7.5)*x;}
//LAZY_BOOLEAN iteration_dom(const REAL& x) {return (x>REAL(0.1) && x <REAL(1));}
LAZY_BOOLEAN iteration_dom(const REAL& x) {return true;}

void itsyst_REAL_with_lipschitz_condition(int count){

  REAL x = 0.5;

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(x,18); rprintf(" %d\n",i);
    }
    x= lipschitz(iteration,iteration_lip,x);
  }
}


void itsyst_REAL_with_lipschitz_condition_and_domain(int count){

  REAL x = 0.5;

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(x,18); rprintf(" %d\n",i);
    }
    x= lipschitz(iteration,iteration_lip,iteration_dom,x);
  }
}

void itsyst_with_INTEGER(int count){


  REAL x = 0.5;

  for ( int i=0; i<=count; i++ ) {
    if ( (i<100) || (i%10)==0 ) {
      rwrite(x,18); rprintf(" %d\n",i);
    }
    x= (x*(1l-x)*375l)/100l;
  }
}


void compute(){
  int test,count;
  rprintf("\nIterated functions system: x = 3.75*x*(1-x)\n");
  rprintf("\nHow to compute (1=double, 2=RATIONAL, 3=REAL, 4=with_INTEGER, 5=with_Lipschitz, 6=with_L+D) : ");
  rscanf("%d",&test);
  rprintf("How many values: ");
  rscanf("%d",&count);
  rprintf("\n");
  switch (test) {
     case 1:;
       itsyst_double(count);
     break;
     case 2:;
       itsyst_RATIONAL(count);
     break;
     case 3:;
       itsyst_REAL(count);
     break;
     case 4:;
       itsyst_with_INTEGER(count);
     break;
     case 5:;
       itsyst_REAL_with_lipschitz_condition(count);
     break;
     case 6:;
       itsyst_REAL_with_lipschitz_condition_and_domain(count);
     break;
  }
  rprintf("\n");
};
